import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.io.*;

class Main extends Frame implements ActionListener {
    static Clipboard mainClipboard = new Clipboard("Main Clipboard");
    Image currentImage;

    Main() {
        super("Transferable Example");
        MenuBar mb = new MenuBar();
        Menu m = new Menu("Edit");

        m.add("Paste");
        m.add("Clear Clipboard");
        m.addActionListener(this);
        mb.add(m);
        setMenuBar(mb);

        setSize(100, 150);
        show();
    }

    public static void main(String[] args) {
        new Main();

        Frame f = new Frame("Image Palette");
        f.add(new ImagePalette(), BorderLayout.CENTER);
        f.pack();
        f.show();
    }

    public void actionPerformed(ActionEvent evt) {
        if (evt.getActionCommand().equals("Paste")) {
            // Get the image in the clipboard.
            Transferable t = mainClipboard.getContents(this);

            try {
                if (t != null) {
                    currentImage = (Image)t.getTransferData(
                        ImageSelection.imageFlavor);
                    repaint();
                }
            } catch (IOException e) {
            } catch (UnsupportedFlavorException e) {
            }
        } else {
            // Clear Clipboard
            mainClipboard.setContents(null, null);
        }
        repaint();
    }

    public void paint(Graphics g) {
        if (currentImage != null) {
            g.drawImage(currentImage, getInsets().left, getInsets().top, this);
        }

        // All containers must call super.paint() in case 
        // it contains lightweight components.
        super.paint(g);
    }
}

class ImagePalette extends Component implements ClipboardOwner {
    // If >= 0, points the currently selected image.
    int curImage = -1;

    // This field holds all the images.
    Image images[] = new Image[4];

    // This field holds the bounds for each image.
    Rectangle rects[] = new Rectangle[4];

    ImagePalette() {
        MediaTracker tracker = new MediaTracker(this);

        // Use a media tracker because we need to know the
        // dimensions of the images immediately.
        for (int i=0; i<images.length; i++) {
            images[i] = getToolkit().getImage("duke"+i+".gif");
            tracker.addImage(images[i], 0);
        }
        try {
            // Wait for images to load.
            tracker.waitForAll();
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Determine the location of each image.
        int x = 0;
        int h = 0;
        for (int i=0; i<images.length; i++) {
            rects[i] = new Rectangle(x, 0, 
                images[i].getWidth(null), images[i].getHeight(null));
            x += rects[i].width;
            h = Math.max(h, rects[i].height);
        }

        addMouseListener(new MouseEventHandler());
        setSize(x, h);
    }

    public void paint(Graphics g) {
        for (int i=0; i<images.length; i++) {
            g.drawImage(images[i], rects[i].x, rects[i].y, this);
            if (i == curImage) {
                // Draw border around current image.
                g.drawRect(rects[i].x, rects[i].y, 
                    rects[i].width, rects[i].height);
            }
        }
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            for (int i=0; i<rects.length; i++) {
                // Find out if an image has been clicked.
                if (rects[i].contains(evt.getX(), evt.getY())) {
                    // Place image into clipboard.
                    Main.mainClipboard.setContents(
                        new ImageSelection(images[i]), ImagePalette.this);

                    curImage = i;
                    repaint();
                    break;
                }
            }
        }
    }

    public void lostOwnership(Clipboard clipboard, Transferable contents) {
        // Remove border around current image.
        curImage = -1;
        repaint();
    }
}

class ImageSelection implements Transferable {
    // Initialize supported flavors.
    public static DataFlavor imageFlavor;

    private Image data;
                                                   
    public ImageSelection(Image data) {
        this.data = data;
    }

    public synchronized DataFlavor[] getTransferDataFlavors() {
        return flavors;
    }

    public boolean isDataFlavorSupported(DataFlavor flavor) {
        return flavor.equals(imageFlavor);
    }

    public synchronized Object getTransferData(DataFlavor flavor) 
                        throws UnsupportedFlavorException, IOException {
        if (flavor.equals(imageFlavor)) {
            return (Object)data;
        } else {
            throw new UnsupportedFlavorException(flavor);
        }
    }

    static {
        try {
            imageFlavor = new DataFlavor(
                Class.forName("java.awt.Image"), "Java Image");
            imageFlavor.setHumanPresentableName("A Java Image Object");
        } catch (Exception e) {
        }
    }

    DataFlavor flavors[] = {imageFlavor};
}
