import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;

class Main extends Frame {
    Main() {
        super("Master");
        setSize(200, 200);
        show();
    }

    public static void main(String[] args) {
        // Create master frame.
        Frame f = new Main();

        // Create slave frame.
        new SlaveFrame(f);
    }
}

class SlaveFrame extends Frame {
    Frame master;
    Point fromMaster;

    SlaveFrame(Frame master) {
        super("Slave");
        this.master = master;

        // Listen for events.
        ComponentEventHandler ceh = new ComponentEventHandler();
        master.addComponentListener(ceh);
        addComponentListener(ceh);

        // Initialize the slave so that on the right side of the frame.
        fromMaster = master.getLocation();
        fromMaster.translate(master.getSize().width, 20);
        setLocation(fromMaster.x, fromMaster.y);
        updateLocation(this);

        setSize(100, 100);
        show();
    }

    void updateLocation(Component componentMoved) {
        Dimension screenSize = getToolkit().getScreenSize();

        if (componentMoved == master) {
            Point newloc = master.getLocation();

            // Determine the new location.
            newloc.translate(fromMaster.x, fromMaster.y);

            // Ensure that slave is within the screen bounds.
            newloc.x = Math.max(0, 
                Math.min(newloc.x, screenSize.width-getSize().width));
            newloc.y = Math.max(0, 
                Math.min(newloc.y, screenSize.height-getSize().height));
    
            // Move the slave to the new location.
            setLocation(newloc.x, newloc.y);
    
            // Correct the offset from master in case it changed.
            fromMaster = master.getLocation();
            fromMaster.translate(newloc.x, newloc.y);

            // Keep the slave above the master.
            toFront();
        } else {
            // Update the offset from master.
            fromMaster = getLocation();
            fromMaster.translate(
                -master.getLocation().x, -master.getLocation().y);
        }
    }

    class ComponentEventHandler extends ComponentAdapter {
        public void componentMoved(ComponentEvent evt) {
            // Master or slave moved.
            updateLocation((Component)evt.getSource());
        }
    }
}
