import java.awt.*;
import java.awt.event.*;
import java.util.*;

class Main extends Frame {
    String keys = "123456789*0#";

    Main() {
        super("KeyEvent Example");
        
        setLayout(new GridLayout(4, 3, 6, 6));

        for (int i=0; i<keys.length(); i++) {
            KeyButton kb = new KeyButton(keys.charAt(i));
        
            kb.addKeyListener(new KeyEventHandler());
            kb.setBackground(Color.pink);
            kb.setForeground(Color.black);
            add(kb);
        }

        setSize(200, 200);
        show();
    }

    class KeyEventHandler extends KeyAdapter {
        public void keyTyped(KeyEvent evt) {
            System.out.println(evt);
        }
    }

    public static void main(String[] args) {
        new Main();
    }
}

class KeyButton extends Component {
    KeyListener keyListener;
    boolean highlighted;
    char key;

    KeyButton(char k) {
        this.key = k;
        addMouseListener(new MouseEventHandler());
    }

    public void paint(Graphics g) {
        int w = getSize().width;
        int h = getSize().height;
        String s = ""+key;
        FontMetrics fm = g.getFontMetrics();

        if (highlighted) {
            g.setColor(getBackground());
            g.fillRoundRect(0, 0, w-1, h-1, 10, 10);
        }
        g.setColor(getForeground());
        g.drawRoundRect(0, 0, w-1, h-1, 10, 10);
        g.drawString(s, (w-fm.stringWidth(s))/2, 
                        (h-fm.getHeight())/2+fm.getAscent());
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            if (keyListener != null) {
                keyListener.keyTyped(
                    new KeyEvent(KeyButton.this, KeyEvent.KEY_TYPED, 
                        System.currentTimeMillis(),
                        0, KeyEvent.VK_UNDEFINED, key));
            }
        }
        public void mouseEntered(MouseEvent evt) {
            highlighted = true;
            repaint();
        }
        public void mouseExited(MouseEvent evt) {
            highlighted = false;
            repaint();
        }
    }

    // Listener support methods
    public synchronized void addKeyListener(KeyListener l) {
        keyListener = AWTEventMulticaster.add(keyListener, l);
    }

    public synchronized void removeKeyListener(KeyListener l) {
        keyListener = AWTEventMulticaster.remove(keyListener, l);
    }  
}
