import java.awt.*;
import java.awt.event.*;

public class Main extends Frame {
    InfestedComponent bugs = new InfestedComponent();

    Main() {
        super("mousePressed Example");

        // Layout component.
        add(bugs, BorderLayout.CENTER);
        pack();
        show();
    }

    public static void main(String args[]) {
        new Main();
    }
}

class InfestedComponent extends Canvas implements Runnable {
    // Bug and field dimensions.  The flower is the same size
    // as a bug.
    int bugSize = 30;
    int fieldSize = 200;

    // true iff a bug is visible and running across the field.
    boolean bugRunning;

    // This field specifies which of the four bug images to use.
    int curImage = 0;

    // This field holds the increments that are applied to the
    // bug's (x, y) location to make it move across the field.
    Point bugDir;

    // Rectangles used the the hit detection code.
    Rectangle bugRect = new Rectangle(fieldSize, 0, bugSize, bugSize);
    Rectangle fieldRect = new Rectangle(0, 0, 200, 200);
    Rectangle[] flowers = new Rectangle[100];

    // Number of flowers on the field.
    int flowerCount;

    // Fields used for double-buffering.
    Image bbuf;
    Graphics bbufG;

    // Color of the field.
    Color bgColor = new Color(204, 255, 204);

    Image bugImages[] = new Image[4];
    Image flowerImage;

    InfestedComponent() {
        // Load images.
        flowerImage = getToolkit().getImage("flower.gif");
        for (int i=0; i<4; i++) {
            bugImages[i] = getToolkit().getImage("bug"+i+".gif");
        }

        // Listen for mouse events.
        addMouseListener(new MouseEventHandler());

        // Start thread.
        (new Thread(this)).start();

        setSize(fieldSize, fieldSize);
    }

    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        // Create the double-buffer if necessary.
        if (bbuf == null) {
            bbuf = createImage(fieldSize, fieldSize);
            bbufG = bbuf.getGraphics();
        }

        // Clear the field.
        bbufG.setColor(bgColor);
        bbufG.fillRect(0, 0, fieldSize, fieldSize);

        // Paint the bug.
        if (bugRunning) {
            bbufG.setColor(Color.black);
            bbufG.drawImage(bugImages[curImage], bugRect.x, bugRect.y, this);
        }

        // Paint flowers.
        bbufG.setColor(Color.yellow);
        for (int i=0; i<flowerCount; i++) {
            bbufG.drawImage(flowerImage, flowers[i].x, flowers[i].y, this);
        }

        // Paint double-buffer on screen.
        g.drawImage(bbuf, 0, 0, this);
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            int x = evt.getX();
            int y = evt.getY();

            // Check if bug is hit.
            if (bugRect.contains(x, y)) {
                // First check that it's not on a flower.
                for (int i=0; i<flowerCount; i++) {
                    if (flowers[i].contains(x, y)) {
                        return;
                    }
                }

                // Bug is hit now turn it into a flower.
                flowers[flowerCount++] = new Rectangle(bugRect);
                bugRunning = false;
                repaint();
            }
        }
    }

    // Convenient integer random number generator.
    int rand(int r) {
        return (int)Math.floor(Math.random() * r);
    }
        
    public void run() {
        int steps = 15;
        int delay = 60;

        while (true) {
            try {
                // Wait a little while before the next bug.
                Thread.sleep(rand(5000));

                // Create a bug.
                bugRect.x = rand(fieldSize-bugSize);
                bugRect.y = rand(fieldSize-bugSize);

                // Pick a direction.
                curImage = rand(4);
                switch (curImage) {
                  case 0:     // from north
                    bugRect.y = 0;
                    bugDir = new Point(0, steps);
                    break;
                  case 1:     // from west
                    bugRect.x = 0;
                    bugDir = new Point(steps, 0);
                    break;
                  case 2:     // from south
                    bugRect.y = fieldSize;
                    bugDir = new Point(0, -steps);
                    break;
                  case 3:     // from east
                    bugRect.x = fieldSize;
                    bugDir = new Point(-steps, 0);
                    break;
                }

                // Start the bug running.
                bugRunning = true;

                while (bugRunning) {
                    // Move the bug.
                    bugRect.x += bugDir.x;
                    bugRect.y += bugDir.y;

                    // Is the bug still on the field?
                    if (!fieldRect.intersects(bugRect)) {
                        bugRunning = false;
                    }
                    repaint();
                    Thread.sleep(delay);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}
