import java.awt.*;
import java.awt.event.*;

class Main extends Frame implements MouseListener {
    // The point where the mouse button was pressed.
    Point curPoint;

    // The current button.
    String curButton;

    Main() {
        super("MouseEvent Example");

        addMouseListener(this);
        setSize(150, 150);
        show();
    }

    // Paint an arrow and the coordinates.
    public void paint(Graphics g) {
        if (curPoint != null) {
            FontMetrics fm = g.getFontMetrics();
            Insets insets = getInsets();

            // Build the arrow.
            Polygon arrow = new Polygon();
            arrow.addPoint(curPoint.x, curPoint.y);
            arrow.addPoint(curPoint.x+9, curPoint.y+18);
            arrow.addPoint(curPoint.x+18, curPoint.y+9);
    
            g.fillPolygon(arrow);
            g.drawString(curButton+"("+curPoint.x+","+curPoint.y+")", 
                curPoint.x+12, curPoint.y+14+fm.getAscent());
        }
    }

    public void mouseClicked(MouseEvent evt) {
        print("MOUSE_CLICKED", evt);
    }
    public void mouseEntered(MouseEvent evt) {
        print("MOUSE_ENTERED", evt);
    }
    public void mouseExited(MouseEvent evt) {
        print("MOUSE_EXITED", evt);
    }
    public void mousePressed(MouseEvent evt) {
        curPoint = evt.getPoint();
        curButton = getCurButton(evt);
        print("MOUSE_PRESSED", evt);
        repaint();
    }
    public void mouseReleased(MouseEvent evt) {
        print("MOUSE_RELEASED", evt);
    }
    
    // Returns a string representing the mouse button that was pressed.
    String getCurButton(MouseEvent evt) {
        String result = "";
        int m = evt.getModifiers();

        if ((m & InputEvent.CTRL_MASK) != 0) {
            result += "c";
        }
        if ((m & InputEvent.SHIFT_MASK) != 0) {
            result += "s";
        }
        if ((m & InputEvent.BUTTON3_MASK) != 0) {
            result += "b3";
        } else if ((m & InputEvent.BUTTON2_MASK) != 0) {
            result += "b2";
        } else if ((m & InputEvent.BUTTON1_MASK) != 0) {
            result += "b1";
        }
        return result;
    }

    // Prints out the details of the mouse event.
    void print(String eventName, MouseEvent evt) {
        String[] modNames = {"alt", "ctl", "meta", "shift", "b1", "b2", "b3"};
        int[] modMasks = {InputEvent.ALT_MASK,
            InputEvent.CTRL_MASK, InputEvent.META_MASK, 
            InputEvent.SHIFT_MASK, InputEvent.BUTTON1_MASK, 
            InputEvent.BUTTON2_MASK, InputEvent.BUTTON3_MASK};
        int m = evt.getModifiers();

        System.out.print(eventName + " (");
        for (int i=0; i<modMasks.length; i++) {
            if ((m & modMasks[i]) != 0) {
                System.out.print(modNames[i]+",");
            }
        }
        System.out.print(") ");
        System.out.print(" point="+evt.getX()+","+evt.getY());
        System.out.print(" clickCount="+evt.getClickCount());
        if (evt.isPopupTrigger()) {
            System.out.print(" isPopupTrigger");
        }
        System.out.println();
    }
    
    public static void main(String args[]) {
        new Main();
    }
}
