import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class Main extends Frame {
    StampCanvas StampCanvas = new StampCanvas();
    Button startButton = new Button("Start");
    Panel palette = new Panel(new FlowLayout());

    Main() {
        super("MouseMotionAdapter Example");

        // Initialize palette with stamps.
        Stamp s;
        palette.add(s = new Stamp(getToolkit().getImage("duke.gif")));
        s.addMouseListener(new MouseEventHandler());
        palette.add(s = new Stamp(getToolkit().getImage("bird.gif")));
        s.addMouseListener(new MouseEventHandler());
        palette.add(s = new Stamp(getToolkit().getImage("house.gif")));
        s.addMouseListener(new MouseEventHandler());

        // Layout components
        add(palette, BorderLayout.SOUTH);
        add(StampCanvas, BorderLayout.CENTER);
        setSize(300, 300);
        show();
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            Stamp stamp = (Stamp)evt.getSource();

            StampCanvas.setStamp(stamp.image);
        }
    }

    public static void main(String args[]) {
        new Main();
    }
}

class Stamp extends Component {
    int width, height;
    Image image;
    
    Stamp(Image image) {
        MediaTracker tracker = new MediaTracker(this);

        this.image = image;
        try {
            tracker.addImage(image, 0);
            tracker.waitForAll();
        } catch (Exception e) {
            e.printStackTrace();
        }
        width = image.getWidth(null);
        height = image.getHeight(null);
    }

    public void paint(Graphics g) {
        g.drawImage(image, 
            (getSize().width-width)/2, (getSize().height-height)/2, this);
    }

    public Dimension getPreferredSize() {
        return new Dimension(width, height);
    }
}

class StampCanvas extends Canvas {
    // Vector of all stamped images.
    Vector stamps = new Vector();

    // Vector of all locations of the stamped images.
    Vector stampPts = new Vector();

    // The current stamp. If null, no stamp is set.
    Image curStamp;

    // The current location of the stamp's origin.
    // If null, don't display the stamp.
    Point curPoint;

    // Double buffer variables
    Image bbuf;
    Graphics bbufG;
    
    StampCanvas() {
        // Listen for events.
        addMouseListener(new MouseEventHandler());
        addMouseMotionListener(new MouseMotionEventHandler());
    }

    void setStamp(Image stamp) {
        curStamp = stamp;
    }

    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        int width = getSize().width;
        int height = getSize().height;

        // Create the double buffer if necessary.
        // Also make it bigger if necessary.
        if (bbuf == null 
              || bbuf.getWidth(null) < width
              || bbuf.getHeight(null) < height) {
            bbuf = createImage(width, height);
            if (bbufG != null) {
                bbufG.dispose();
            }
            bbufG = bbuf.getGraphics();
        }

        // Clear the background.
        bbufG.setColor(Color.lightGray);
        bbufG.fillRect(0, 0, width, height);

        // Paint all the stamps.
        for (int i=0; i<stamps.size(); i++) {
            Image stamp = (Image)stamps.elementAt(i);
            Point pt = (Point)stampPts.elementAt(i);

            bbufG.drawImage(stamp, pt.x, pt.y, this);
        }

        // Paint the current stamp.
        if (curStamp != null && curPoint != null) {
            bbufG.drawImage(curStamp, curPoint.x, curPoint.y, this);
        }

        // Draw the off-screen image to the display.
        g.drawImage(bbuf, 0, 0, this);
    }

    // Handler for mouse move events.
    class MouseMotionEventHandler extends MouseMotionAdapter {
        public void mouseMoved(MouseEvent evt) {
            if (curStamp != null) {
                int x = evt.getX()-curStamp.getWidth(null)/2;
                int y = evt.getY()-curStamp.getHeight(null)/2;

                // Keep the image with the bounds of the canvas.
                x = Math.max(0, Math.min(
                    x, getSize().width-curStamp.getWidth(null)));
                y = Math.max(0, Math.min(
                    y, getSize().height-curStamp.getHeight(null)));

                curPoint = new Point(x, y);
                repaint();
            }
        }
    }

    // Handler for mouse pressed and exit events.
    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            // Save the stamp.
            if (curStamp != null) {
                stamps.addElement(curStamp);
                stampPts.addElement(new Point(curPoint.x, curPoint.y));
                repaint();
            }
        }
        public void mouseExited(MouseEvent evt) {
            curPoint = null;
            repaint();
        }
    }
}

