import java.awt.*;
import java.awt.event.*;
import java.util.*;

class Main extends Frame implements ActionListener {
    FrameManager frameManager = new FrameManager();

    Main() {
        super("WindowEvent Example");
        Button b;
        Panel p = new Panel(new GridLayout(0, 1));

        // Populate the control panel.
        p.add(b = new Button("New"));
        b.addActionListener(this);
        p.add(b = new Button("Show/Hide"));
        b.addActionListener(this);
        p.add(b = new Button("Close"));
        b.addActionListener(this);

        add(p, BorderLayout.EAST);
        add(frameManager, BorderLayout.CENTER);
        frameManager.addActionListener(this);

        setSize(200, 100);
        show();
    }

    // Button actions.
    public void actionPerformed(ActionEvent evt) {
        if (evt.getActionCommand().equals("New")) {
            new MyFrame(frameManager);
            return;
        }

        MyFrame f = frameManager.getFrame();
        if (f == null) {
            return;
        }
        if (evt.getActionCommand().equals("Show/Hide")
                   || evt.getSource() == frameManager) {
            if (f.isShowing()) {
                f.setVisible(false);
            } else {
                f.show();
            }
            frameManager.update(f);
        } else if (evt.getActionCommand().equals("Close")) {
            frameManager.close(f);
            f.dispose();
        }
    }

    public static void main(String args[]) {
        new Main();
    }
}

class MyFrame extends Frame implements WindowListener {
    FrameManager frameManager;
    static int count;    // for naming the frames.
    boolean iconified;
    boolean activated;

    MyFrame(FrameManager wl) {
        super("Frame"+(count++));
        frameManager = wl;

        // Add this frame to the window list.
        frameManager.add(this);

        // Listen for events.
        addWindowListener(this);
        addComponentListener(new ComponentEventHandler());

        add(new Label(getTitle(), Label.CENTER), BorderLayout.CENTER);
        setSize(100, 50);
    }

    // Window Events
    public void windowOpened(WindowEvent evt) {
        System.out.println("WINDOW_OPENED");
    }
    public void windowClosing(WindowEvent evt) {
        dispose();
    }
    public void windowClosed(WindowEvent evt) {
        frameManager.close((MyFrame)evt.getSource());
    }
    public void windowIconified(WindowEvent evt) {
        iconified = true;
        frameManager.update(this);
    }
    public void windowDeiconified(WindowEvent evt) {
        iconified = false;
        frameManager.update(this);
    }
    public void windowActivated(WindowEvent evt) {
        activated = true;
        frameManager.update(this);
    }
    public void windowDeactivated(WindowEvent evt) {
        activated = false;
        frameManager.update(this);
    }

    // Component Events
    class ComponentEventHandler extends ComponentAdapter {
        public void componentShown(ComponentEvent evt) {
            frameManager.update((MyFrame)evt.getSource());
        }
        public void componentHidden(ComponentEvent evt) {
            frameManager.update((MyFrame)evt.getSource());
        }
    }
}

class FrameManager extends List {
    // This holds a list of frames.  The order of the frames 
    // in this vector corresponds to the order of the frames
    // in the list component.
    Vector frames = new Vector();

    FrameManager() {
        setFont(new Font("Monospaced", Font.PLAIN, 12));
    }

    // Returns the currently selected frame or null if there
    // is no selected item.
    MyFrame getFrame() {
        int ix = getSelectedIndex();

        if (ix >= 0) {
            return (MyFrame)frames.elementAt(ix);
        }
        return null;
    }

    void add(MyFrame f) {
        add(f.getTitle());
        frames.addElement(f);
        update(f);
    }

    void close(MyFrame f) {
        int ix = frames.indexOf(f);

        if (ix >= 0) {
            frames.removeElementAt(ix);
            remove(ix);
        }
    }

    void update(MyFrame f) {
        int ix = frames.indexOf(f);
    
        if (ix >= 0) {
            String s = f.getTitle() + "  ";

            if (f.isShowing()) {
                if (f.iconified) {
                    s += '-';
                } else {
                    s += '+';
                }
                if (f.activated) {
                    s += '*';
                } else {
                    s += ' ';
                }
            }
            replaceItem(s, ix);
        }
    }
}