import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;

class Main extends Frame implements ActionListener {
    TextField textField = new TextField();
    ImageCanvas icv;

    Main(String filename) {
        super("DirectColorModel Example");
        try {
            // Retrieve the image.
            Image image = getToolkit().getImage(filename);

            add(icv = new ImageCanvas(image), BorderLayout.CENTER);
            add(textField, BorderLayout.SOUTH);
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Add listener for text field
        textField.addActionListener(this);
        setSize(50, 100);
        show();
    }

    public void actionPerformed(ActionEvent evt) {
        icv.setColorBits(Integer.parseInt(textField.getText()));
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Component {
    Image newImage;
    Image image;
    LowResFilter imgf = new LowResFilter();

    ImageCanvas(Image image) {
        this.image = image;
        processImage();
    }

    void setColorBits(int bits) {
        imgf.setColorBits(bits);
        processImage();
    }

    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        g.drawImage(newImage, 0, 0, this);
    }

    void processImage() {
        ImageProducer ip = image.getSource();

        ip = new FilteredImageSource(ip, imgf);
        newImage = getToolkit().createImage(ip);
        repaint();
    }
}

class LowResFilter extends RGBImageFilter {
    int bits;    // bits for each color.
    DirectColorModel lowResColorModel = 
        (DirectColorModel)ColorModel.getRGBdefault();

    void setColorBits(int bits) {
        int mask = 0;
        this.bits = bits;

        for (int i=0; i<bits; i++) {
            mask |= (mask<<1) + 1;
        }
        lowResColorModel = new DirectColorModel(bits * 3, 
            mask<<(2 * bits), mask<<bits, mask);
        System.out.println(
            Integer.toString(lowResColorModel.getAlphaMask(), 16));
        System.out.println(
            Integer.toString(lowResColorModel.getRedMask(), 16));
        System.out.println(
            Integer.toString(lowResColorModel.getGreenMask(), 16));
        System.out.println(
            Integer.toString(lowResColorModel.getBlueMask(), 16));
    }

    public void setColorModel(ColorModel model) {
        consumer.setColorModel(lowResColorModel);
    }

    public int filterRGB(int x, int y, int rgb) {
        int res = 1 << bits;
        int a = ColorModel.getRGBdefault().getAlpha(rgb);
        int r = ColorModel.getRGBdefault().getRed(rgb);
        int g = ColorModel.getRGBdefault().getGreen(rgb);
        int b = ColorModel.getRGBdefault().getBlue(rgb);

        return ((a * res / 256) << 3*bits)
            | ((r * res / 256) << 2*bits)
            | ((g * res / 256) << bits)
            | ((b * res / 256));
    }

    public void filterRGBPixels(int x, int y, int w, int h,
                                int pixels[], int off, int scansize) {
        int index = off;
        for (int cy = 0; cy < h; cy++) {
            for (int cx = 0; cx < w; cx++) {
                pixels[index] = filterRGB(x + cx, y + cy, pixels[index]);
                index++;
            }
            index += scansize - w;
        }
        consumer.setPixels(x, y, w, h, lowResColorModel, pixels, off, scansize);
    }
}

