import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.*;

class Main extends Frame {
    Main(String filename) {
        super("canFilterIndexColorModel Example");
        add(new ImageCanvas(getToolkit().getImage(filename)), 
            BorderLayout.CENTER);

        setSize(50, 100);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Component {
    Image brightImage;
    Image image;
    BrightnessFilter imgf = new BrightnessFilter();

    ImageCanvas(Image image) {
        this.image = image;
        processImage();
        addKeyListener(new KeyEventHandler());
    }

    public void paint(Graphics g) {
        g.drawImage(brightImage, 0, 0, this);
    }

    void processImage() {
        ImageProducer ip = image.getSource();

        ip = new FilteredImageSource(ip, imgf);
        brightImage = getToolkit().createImage(ip);
        repaint();
    }

    class KeyEventHandler extends KeyAdapter {
        public void keyTyped(KeyEvent evt) {
            char key = evt.getKeyChar();
            if (key == '+') {
                imgf.addBrightness(.1f);
            } else if (key == '-') {
                imgf.addBrightness(-.1f);
            }
            processImage();
        }
    }
}

class BrightnessFilter extends RGBImageFilter {
    float bDelta;
    float[] hsb = new float[3];
    int width, height;

    BrightnessFilter() {
        canFilterIndexColorModel = true;
    }

    public void setDimensions(int w, int h) {
        super.setDimensions(w, h);
        width = w;
        height = h;
    }

    public void addBrightness(float f) {
        bDelta += f;
    }

    public int filterRGB(int x, int y, int rgb) {
        // x and y are both -1
        Color c = new Color(rgb);
        Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), hsb);
        hsb[2] = Math.max(0.0f, Math.min(1.0f, hsb[2] + bDelta));
        return Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]);
    }
}

