import java.awt.*;
import java.awt.event.*;

class Main extends Frame implements AdjustmentListener {
    int r = 255;
    int g = 255;
    int b = 255;
    Scrollbar sbRed = new Scrollbar(Scrollbar.VERTICAL, 0, 1, 1, 257);
    Scrollbar sbGreen = new Scrollbar(Scrollbar.VERTICAL, 0, 1, 1, 257);
    Scrollbar sbBlue = new Scrollbar(Scrollbar.VERTICAL, 0, 1, 1, 257);
    Scrollbar sbH = new Scrollbar(Scrollbar.VERTICAL, 0, 1, 1, 1002);
    Scrollbar sbS = new Scrollbar(Scrollbar.VERTICAL, 0, 1, 1, 1002);
    Scrollbar sbB = new Scrollbar(Scrollbar.VERTICAL, 0, 1, 1, 1002);
    TextField status = new TextField();
    float[] hsb;

    Main() {
        super("Color Example");
        sbRed.addAdjustmentListener(this);
        sbGreen.addAdjustmentListener(this);
        sbBlue.addAdjustmentListener(this);
        sbH.addAdjustmentListener(this);
        sbS.addAdjustmentListener(this);
        sbB.addAdjustmentListener(this);

        add(makeScrollbars("R", sbRed, "G", sbGreen, "B", sbBlue), 
            BorderLayout.WEST); 
        add(makeScrollbars("H", sbH, "S", sbS, "B", sbB), BorderLayout.EAST);
        status.setEditable(false);
        add(status, BorderLayout.SOUTH); 
        setSize(300, 300);
        show();
        setBackground(adjustHSBScrollbars());
        adjustRGBScrollbars();
    }

    Panel makeScrollbars(String l1, Scrollbar sb1,
            String l2, Scrollbar sb2, String l3, Scrollbar sb3) {
        double[] rowWeights = {0.0, 1.0};
        GridBagLayout gbl = new GridBagLayout();
        Panel p = new Panel(gbl);

        gbl.rowWeights = rowWeights;
        p.setLayout(gbl);
        add(p, gbl, new Label(l1, Label.CENTER), 
            0, 0, GridBagConstraints.NONE);
        add(p, gbl, sb1, 0, 1, GridBagConstraints.VERTICAL);
        add(p, gbl, new Label(l2, Label.CENTER), 
            1, 0, GridBagConstraints.NONE);
        add(p, gbl, sb2, 1, 1, GridBagConstraints.VERTICAL);
        add(p, gbl, new Label(l3, Label.CENTER), 
            2, 0, GridBagConstraints.NONE);
        add(p, gbl, sb3, 2, 1, GridBagConstraints.VERTICAL);
        return p;
    }

    void add(Panel p, GridBagLayout gbl, Component comp, int x, int y, int fill) {
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.fill = fill;
        gbl.setConstraints(comp, gbc);
        p.add(comp);
    }

    public void adjustmentValueChanged(AdjustmentEvent evt) {
        Color c = getBackground();
        Object src = evt.getSource();

        // Value from scrollbar is one greater than actual RGB or HSB value

        if (src == sbRed) {
            r = 255-(sbRed.getValue()-1);
            c = adjustHSBScrollbars();
        } else if (src == sbGreen) {
            g = 255-(sbGreen.getValue()-1);
            c = adjustHSBScrollbars();
        } else if (src == sbBlue) {
            b = 255-(sbBlue.getValue()-1);
            c = adjustHSBScrollbars();
        } else if (src == sbH) {
            hsb[0] = (1000-(sbH.getValue()-1)) / 1000.0f;
            c = adjustRGBScrollbars();
        } else if (src == sbS) {
            hsb[1] = (1000-(sbS.getValue()-1)) / 1000.0f;
            c = adjustRGBScrollbars();
        } else if (src == sbB) {
            hsb[2] = (1000-(sbB.getValue()-1)) / 1000.0f;
            c = adjustRGBScrollbars();
        }
        setBackground(c);
        repaint();
        status.setText("RGB("+r+","+g+","+b+"  /  "+
            "#"+Integer.toString(c.getRGB()&0xffffff, 16)+")"+
            "    HSB("+hsb[0]+","+hsb[1]+","+hsb[2]+")");
    }

    Color adjustHSBScrollbars() {
        hsb = Color.RGBtoHSB(r, g, b, null);
        // need to offset values by 1 to account for 'visible' part
        // of scrollbar
        sbH.setValue(1001-(int)(hsb[0] * 1000));
        sbS.setValue(1001-(int)(hsb[1] * 1000));
        sbB.setValue(1001-(int)(hsb[2] * 1000));
        return new Color(r, g, b);
    }

    Color adjustRGBScrollbars() {
        Color c = Color.getHSBColor(hsb[0], hsb[1], hsb[2]);

        // An alternate way of converting the HSB values to RGB:
        // Color c = new Color(Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]));

        // need to offset values by 1 to account for 'visible' part
        // of scrollbar
        sbRed.setValue(256-(r = c.getRed()));
        sbGreen.setValue(256-(g = c.getGreen()));
        sbBlue.setValue(256-(b = c.getBlue()));
        return c;
    }

    public static void main(String[] args) {
        new Main();
    }
}

