import java.awt.*;
import java.awt.event.*;

class Main extends Frame {
    // Returns a random number in the range [0 .. r)
    static int random(int r) {
        return (int)Math.floor(Math.random()*r);
    }

    public static void main(String[] args) {
        Frame f = new Frame("setClip Example");
        Polygon polygon = new Polygon();

        // Create some random points.  The points are no
        // more than 100 pixels from the origin.
        for (int i=0; i<25; i++) {
            polygon.addPoint(random(200)-100, random(200)-100);
        }

        // Layout and show components.
        f.add(new PolygonRotator(polygon), BorderLayout.CENTER);
        f.setSize(200, 200);
        f.show();
    }
}

class PolygonRotator extends Canvas implements Runnable {
    Polygon origPoly;
    Polygon curPoly;
    Rectangle view = new Rectangle(0, 0, 100, 100);

    PolygonRotator(Polygon polygon) {
        origPoly = polygon;

        // Listen for mouse events.
        addMouseListener(new MouseEventHandler());
        addMouseMotionListener(new MouseMotionEventHandler());

        // Start the rotating thread.
        (new Thread(this)).start();
    }

    public void paint(Graphics g) {
        // Draw an outline around the view.
        g.drawRect(view.x-1, view.y-1, view.width+1, view.height+1);

        // Set the clipping area.
        g.setClip(view.x, view.y, view.width, view.height);

        // The following would have worked just as well.
        // g.clipRect(view.x, view.y, view.width, view.height);

        // Now paint the polygon.
        if (curPoly != null) {
            // Place the polygon at the center of the canvas.
            g.translate(getSize().width/2, getSize().height/2);
            g.drawPolygon(curPoly);
        }
    }

    // Returns a rotated copy of p at the angle a.
    Polygon rotate(Polygon p, double a) {
        Polygon q = new Polygon();
        q.xpoints = new int[p.npoints];
        q.ypoints = new int[p.npoints];
        q.npoints = p.npoints;

        for (int i=0; i<p.npoints; i++) {
            q.xpoints[i] = 
                (int)(p.xpoints[i]*Math.cos(a)-p.ypoints[i]*Math.sin(a));
            q.ypoints[i] = 
                (int)(p.xpoints[i]*Math.sin(a)+p.ypoints[i]*Math.cos(a));
        }
        return q;
    }

    // If non-null, location of the down click.
    Point mouseDown;
    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            if (view.contains(evt.getX(), evt.getY())) {
                // Record the down click on if inside view.
                mouseDown = evt.getPoint();
                mouseDown.translate(-view.x, -view.y);
            }
        }
        public void mouseReleased(MouseEvent evt) {
            mouseDown = null;
        }
    }

    class MouseMotionEventHandler extends MouseMotionAdapter {
        public void mouseDragged(MouseEvent evt) {
            if (mouseDown != null) {
                // Move the view.
                view.setLocation(evt.getX()-mouseDown.x, 
                                 evt.getY()-mouseDown.y);
                repaint();
            }
        }
    }

    public void run() {
        double angle = 0.0;

        try {
            while (true) {
                // Create a rotated version of the original polygon.
                curPoly = rotate(origPoly, angle);
    
                // Change the angle at .1 radian every tick.
                angle += .1;
                repaint();
                Thread.sleep(100);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
