import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

class Main extends Frame {
    Main(String filename) {
        super("getGraphics Example");
        // Retrieve the image.
        add(new ImageCanvas(
            getToolkit().getImage(filename)), 
            BorderLayout.CENTER); 
        setSize(200, 200);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Component {
    Image image;
    Image backBuffer;
    Graphics backBufferG;

    ImageCanvas(Image image) {
        this.image = image;
        prepareImage(image, this);
        addMouseMotionListener(new MouseMotionHandler());
    }

    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        if (backBuffer == null) {
            if (g.drawImage(image, 0, 0, this) && backBuffer == null) {
                int w = image.getWidth(this);
                int h = image.getHeight(this);

                backBuffer = createImage(w, h);
                backBufferG = backBuffer.getGraphics();
                backBufferG.setColor(getBackground());
                backBufferG.fillRect(0, 0, w, h);
                backBufferG.drawImage(image, 0, 0, this);
            }
            return;
        }
        g.drawImage(backBuffer, 0, 0, this);
    }

    class MouseMotionHandler extends MouseMotionAdapter {
        public void mouseDragged(MouseEvent evt) {
            backBufferG.setColor(Color.red);
            backBufferG.fillOval(evt.getX(), evt.getY(), 5, 5);
            repaint();
        }
    }
}
