import java.awt.*;
import java.awt.event.*;

class Main extends Frame implements ActionListener {
    TextField inputTf = new TextField();
    Label statusBar = new Label("", Label.CENTER);
    CharDisplay charDisplay = new CharDisplay(statusBar);

    Main() {
        // Set up listeners.
        inputTf.addActionListener(this);

        // Initialize status bar.
        statusBar.setForeground(Color.white);
        statusBar.setBackground(Color.black);

        // Layout components.
        add(statusBar, BorderLayout.NORTH);
        add(charDisplay, BorderLayout.CENTER);
        add(inputTf, BorderLayout.SOUTH);
        setSize(400, 400);
        show();
    }
    
    char oldCode = (char)-1;
    public void actionPerformed(ActionEvent evt) {
        // Change starting code.
        char code = (char)Integer.parseInt(inputTf.getText(), 16);

        if (code == oldCode) {
            code = charDisplay.nextPage();
        } else {
            charDisplay.setStartCode(code);
        }
        inputTf.setText(Integer.toHexString(code).toUpperCase());
        oldCode = code;
    }

    public static void main(String[] args) {
        new Main();
    }
}

class CharDisplay extends Canvas {
    char startCode;
    Label statusBar;
    int charW;
    int charH;
    int charsPerLine = 32;

    Font font = new Font("Monospaced", Font.PLAIN, 14);
    FontMetrics fontM;

    CharDisplay(Label statusBar) {
        this.statusBar = statusBar;

        // Set up font information.
        setFont(font);
        fontM = getFontMetrics(font);
        charW = fontM.getMaxAdvance();
        charH = fontM.getHeight();

        // Listen for mouse events.
        addMouseListener(new MouseEventHandler());
        addMouseMotionListener(new MouseMotionEventHandler());
    }

    // Sets the starting code.
    void setStartCode(char startCode) {
        this.startCode = startCode;
        repaint();
    }

    char nextPage() {
        startCode += charsPerLine * getSize().height/charH;
        repaint();
        return startCode;
    }

    public void paint(Graphics g) {
        int s = startCode;
        int y = 0;
        int lines = getSize().height/fontM.getHeight();
        char[] buf = new char[1];
        Rectangle clip = g.getClipBounds();

        // The following could also be used to retrieve the clipping area.
        // Rectangle clip = g.getClip();
    
        // Grow the clipping area to include surrounding characters.
        if (clip != null) {
            clip.x -= charW;
            clip.width += 2 * charW;
            clip.y -= charH;
            clip.height += 2 * charH;
        }

        for (int i=0; i<lines; i++) {
            // Fill one line with character.
            for (int j=0; j<charsPerLine; j++) {
                buf[0] = (char)s++;
                // Don't paint the character if not in the clipping area.
                if (clip == null || clip.contains(j*charW, y)) {
                    g.drawChars(buf, 0, 1, j*charW, y+fontM.getAscent());
                }
            }
            y += charH;
        }
    }

    class MouseEventHandler extends MouseAdapter {
        Window window;

        public void mousePressed(MouseEvent evt) {
            char code = (char)(evt.getX()/charW 
                + evt.getY()/charH * charsPerLine
                + startCode);

            Component c = CharDisplay.this;
            while (!(c instanceof Frame)) {
                c = c.getParent();
            }

            // Display the window
            Point pt = getLocationOnScreen();
            pt.translate(evt.getX(), evt.getY());
            window = new LargeChar(code, (Frame)c, pt);
        }
        public void mouseReleased(MouseEvent evt) {
            window.dispose();
        }
    }

    class MouseMotionEventHandler extends MouseMotionAdapter {
        public void mouseMoved(MouseEvent evt) {
            int code = evt.getX()/charW 
                + evt.getY()/charH * charsPerLine
                + startCode;

            statusBar.setText(Integer.toHexString(code).toUpperCase());
        }
    }
}

class LargeChar extends Window {
    FontMetrics fontM;
    char ch;
    
    LargeChar(char ch, Frame frame, Point pt) {
        super(frame);
        this.ch = ch;

        Font font = new Font("Monospaced", Font.BOLD, 72);
        fontM = getFontMetrics(font);
        setFont(font);

        // Listen for mouse events.
        addMouseListener(new MouseEventHandler());

        // Set bounds and show.
        setSize(fontM.charWidth(ch)+2, fontM.getHeight()+2);
        setLocation(pt.x-getSize().width/2, pt.y-getSize().height/2);
        show();
    }

    public void paint(Graphics g) {
        g.drawString(""+ch, 1, fontM.getAscent()+1);
        g.drawRect(0, 0, getSize().width-1, getSize().height-1);
    }

    class MouseEventHandler extends MouseAdapter {
        public void mouseReleased(MouseEvent evt) {
            dispose();
        }
    }
}
