import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;

class Main extends Frame {
    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println(
                "Usage: java Main <name of component bean class file>");
            System.exit(1);
        }

        try {
            // Load the bean class.
            // (See the Introspector class example for FileClassLoader source.)
            Class beanClass = FileClassLoader.load(args[0]);
            BeanInfo beanInfo = Introspector.getBeanInfo(beanClass);

            // Get the event set descriptors, looking for one that fires
            // action events.
            EventSetDescriptor eds[] = beanInfo.getEventSetDescriptors();
            for (int i=0; i<eds.length; i++) {
                if (eds[i].getListenerType() == 
                        java.awt.event.ActionListener.class) {
                    // Instantiate the bean.
                    Object bean = beanClass.newInstance();

                    // Put the bean in a frame.
                    new TestFrame((Component)bean, eds[i]);
                    break;
                }
            }
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (IntrospectionException e) {
            e.printStackTrace();
        }
    }
}

class TestFrame extends Frame implements ActionListener {
    Component bean;
    EventSetDescriptor esd;

    TestFrame(Component bean, EventSetDescriptor esd) {
        super("getAddListenerMethod Example");
        Method adder = esd.getAddListenerMethod();
        this.bean = bean;
        this.esd = esd;

        // Register with the bean for action events.
        Object[] adderArgs = {this};
        try {
            adder.invoke(bean, adderArgs);
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }

        // Watch for window closing events.
        addWindowListener(new WindowEventHandler());

        // Layout and show bean.
        add((Component)bean, BorderLayout.CENTER);
        setSize(100, 100);
        show();
    }

    // Window events.

    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent evt) {
            // Get remove listener method.
            Method remover = esd.getRemoveListenerMethod();
            
            // Remove frame from listening for action events.
            Object[] removerArgs = {TestFrame.this};
            try {
                remover.invoke(bean, removerArgs);
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }

            // Destroy the frame.
            dispose();
        }
    }

    // Action events.
    public void actionPerformed(ActionEvent evt) {
        System.out.println(evt);
    }
}
