import java.awt.*;
import java.beans.*;

class Main implements VetoableChangeListener {
    Main() {
        Bean bean = null;

        try {
            // Create the bean.
            bean = (Bean)Beans.instantiate(Main.class.getClassLoader(), "Bean");
        } catch (Exception e) {
            e.printStackTrace();
        }
        bean.addVetoableChangeListener(this);
    }

    // Print the new price.  If too high, veto it.
    public void vetoableChange(PropertyChangeEvent evt) 
        throws PropertyVetoException {
        float newPrice = ((Float)evt.getNewValue()).floatValue();

        System.out.print(evt.getPropertyName() + ": " + newPrice);

        // Don't let the price go up to more than $2.50
        if (newPrice > 2.50f) {
            System.out.println(" VETO");
            throw(new PropertyVetoException("Too Much!", evt));
        } else {
            System.out.println();
        }
    }

    public static void main(String args[]) {
        new Main();
    }
}

public class Bean implements Runnable {
    float price = 1.49f;

    // Constructor
    public Bean() {
        (new Thread(this)).start();
    }

    // The price is a constrained property.
    public float getPrice() {
        return price;
    }

    public void setPrice(float newPrice) {
        try {
            vceListeners.fireVetoableChange("price", 
                new Float(price), new Float(newPrice));
            price = newPrice;
        } catch (PropertyVetoException e) {
            // Don't change the price
            PropertyChangeEvent pce = e.getPropertyChangeEvent();
            System.out.print("Some listener didn't like the new price "
                + ((Float)pce.getNewValue()).floatValue());
            System.out.println(" - " + e.getMessage());
        }
    }

    // Create the listener list.
    VetoableChangeSupport vceListeners = 
        new VetoableChangeSupport(this);

    // The listener list wrapper methods.
    public synchronized void addVetoableChangeListener(
                                VetoableChangeListener l) {
        vceListeners.addVetoableChangeListener(l);
    }
    public synchronized void removeVetoableChangeListener(
                                VetoableChangeListener l) {
        vceListeners.removeVetoableChangeListener(l);
    }

    public void run() {
        try {
            while (true) {
                // Change price up or down by .50.
                setPrice(Math.max(1.0f, price + .5f - (float)Math.random()));

                Thread.sleep(500);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
